//
//  BonanzaAppDelegate.h
//  Bonanza
//
//  Created by 永田 央 on 12/03/24.
//  Copyright 2012 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

typedef struct BonMoveInfo {
	unsigned char bw;      /*  black: 0, white: 1  */
	unsigned char koma;    /*  koma code; 99 for end of game, 98 for mate  */
	unsigned char capture; /*  captured koma by this move  */
	unsigned char promote; /*  no promote: 0, promote: 1  */
	unsigned char pos[4];  /*  old column, old row, new column, new row  */
	int moveTime; /*  elapsed time for move  */
	short moves;  /*  tekazu  */
	char mes[32]; /*  message  */
	int b_elapse, w_elapse;  /*  elapsed time for black and white  */
} BonMoveInfo;

@interface BonanzaAppDelegate : NSObject {
	NSTask *bonTask;
	NSPipe *pipeToBon, *pipeFromBon;
	NSString *bonPath;
	NSMutableArray *bonMoves;
	NSString *bonLastUnfinishedMessage;
	int bonMoveIndex;
	
    IBOutlet NSWindow *window;
	IBOutlet id shogiView;
	IBOutlet NSWindow *promoteWindow;
	IBOutlet NSWindow *settingsWindow;
	IBOutlet NSWindow *gameOverWindow;
	IBOutlet NSWindow *aboutWindow;
	IBOutlet NSTextField *blackName;
	IBOutlet NSTextField *whiteName;
	IBOutlet NSTextField *blackTebanInfo;
	IBOutlet NSTextField *whiteTebanInfo;
	IBOutlet NSTextField *blackElapsedTime;
	IBOutlet NSTextField *whiteElapsedTime;
	IBOutlet NSTextField *blackChecked;
	IBOutlet NSTextField *whiteChecked;
	IBOutlet NSButton *settingsButton;
	IBOutlet NSButton *startButton;
	IBOutlet NSButton *undoButton;
	IBOutlet NSButton *resignButton;
	IBOutlet NSTextField *winText;
	IBOutlet NSButton *aboutOKButton;
	IBOutlet NSTextField *aboutVersionText;
	IBOutlet NSTextField *aboutBuildDateText;
	
	/*  1:fu, 2:kyo, 3:kei, 4:gin, 5:kin, 6:kaku, 7:hisha, 8:ou, +10:nari
	    positive:black, negative:white  */
	signed char board[81];
	unsigned char mochigoma[2][8]; /* mochigoma[0][0] is number of fu's for black, etc. */
	int handicap;
	NSString *handicapSpec;  /*  Handicap specification for bonanza  */
	NSString *bonanzaInitString;    /*  Initialize string (for easy restarting)  */

	/*  Koma to move  */
	BonMoveInfo moving;
	unsigned char board_focus[81]; /* 1: koma to move, 2: movable position */
	NSPoint draggingKomaPosition;
	int draggingMochigomaIndex;
	BOOL isDraggingKoma;
	
	NSImage *images[2][18];
	NSImage *banImage, *gridImage, *selfFocusImage, *movableFocusImage;
	
	int teban;  /*  0: black, 1: white, -1: none  */
	int status;  /*  1: black resigned, 2: white resigned, 3: black not resigned but mated, 4: white not resigned but mated, 5: black is checked, 6: white is checked  */
	
	/*  Bonanza assumes that black moves first. However, for handicapped games,
	    it is more common for white to move first. This flag fakes bonanza by
	    reversing black/white and all coordinates.  */
	BOOL whiteMovesFirst;
	
	/*  YES if white is displayed in 'this side', NO otherwise  */
	BOOL whiteIsThisSide;
	
	/*  YES if the player is human, NO if the player is Bonanza  */
	BOOL isHuman[2];
	
	/*  Count the thinking time  */
	NSTimer *timer;
	int elapsedTime[2];
	int lastMoveTime;
}
- (IBAction)okPressed:(id)sender;
- (IBAction)cancelPressed:(id)sender;
- (IBAction)openSetUpSheet:(id)sender;
- (IBAction)okSetUpSheet:(id)sender;
- (IBAction)cancelSetUpSheet:(id)sender;
- (IBAction)startNewGame:(id)sender;
- (IBAction)undoMove:(id)sender;
- (IBAction)resignGame:(id)sender;
- (IBAction)okGameOverSheet:(id)sender;
- (IBAction)okAboutWindow:(id)sender;
- (IBAction)showAboutWindow:(id)sender;
- (void)stopBonanza;
- (void)startBonanza;
- (void)stopTimer;
- (void)startTimer;
- (void)updateTebanInfo;
- (void)displayBoardInView:(NSView *)boardView;
- (void)makeMove:(const BonMoveInfo *)iptr withAnimation:(BOOL)showAnimation;
- (void)mouseDown:(NSEvent *)theEvent inView:(NSView *)boardView;
- (void)mouseDragged:(NSEvent *)theEvent inView:(NSView *)boardView;
- (void)mouseUp:(NSEvent *)theEvent inView:(NSView *)boardView;
- (void)mouseMoved:(NSEvent *)theEvent inView:(NSView *)boardView;
@end
