/*
 *  endian.c
 *  Bonanza
 *
 *  Created by 永田 央 on 12/03/24.
 *  Copyright 2012 __MyCompanyName__. All rights reserved.
 *
 */

#include "shogi.h"
#include <stdio.h>
#include <stdint.h>

#if 1 || __BIG_ENDIAN__

size_t
fread_var(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream)
{
	char *p = (char *)ptr;
	size_t retval = fread(ptr, size, nitems, stream);
	size_t offset;
	if (size == 1 || retval == 0)
		return retval;
	if (size == 2) {
		for (offset = 0; offset < retval; offset++, p += 2)
			*((uint16_t *)p) = SwapInt16LittleToHost(*((uint16_t *)p));
	} else if (size == 4) {
		for (offset = 0; offset < retval; offset++, p += 4)
			*((uint32_t *)p) = SwapInt32LittleToHost(*((uint32_t *)p));
	} else if (size == 8) {
		for (offset = 0; offset < retval; offset++, p += 8)
			*((uint64_t *)p) = SwapInt64LittleToHost(*((uint64_t *)p));
	}
	return retval;
}

#define ALLOC_SIZE 65536
static inline size_t
swap_fwrite(char *cp, size_t size, size_t nitems, FILE *restrict stream)
{
	size_t offset;
	char *p = cp;
	if (size == 2) {
		for (offset = 0; offset < nitems; offset++, p += 2)
			*((uint16_t *)p) = SwapInt16HostToLittle(*((uint16_t *)p));
	} else if (size == 4) {
		for (offset = 0; offset < nitems; offset++, p += 4)
			*((uint32_t *)p) = SwapInt32HostToLittle(*((uint32_t *)p));
	} else if (size == 8) {
		for (offset = 0; offset < nitems; offset++, p += 8)
			*((uint64_t *)p) = SwapInt64HostToLittle(*((uint64_t *)p));
	}
	return fwrite(cp, size, nitems, stream);
}

size_t
fwrite_var(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream)
{
	if (size == 1)
		return fwrite(ptr, size, nitems, stream);
	else {
		size_t write_nitems = 0;
		char *cp = alloca(ALLOC_SIZE);
		size_t alloc_nitems = ALLOC_SIZE / size;
		while (nitems > 0) {
			size_t retval;
			size_t temp_nitems = (nitems > alloc_nitems ? alloc_nitems : nitems);
			memmove(cp, (const char *)ptr, temp_nitems * size);
			retval = swap_fwrite(cp, size, temp_nitems, stream);
			write_nitems += retval;
			if (retval < temp_nitems)
				return write_nitems;
			nitems -= temp_nitems;
			ptr = (const void *)((const char *)ptr + temp_nitems * size);
		}
		return write_nitems;
	}
}

#endif /* __BIG_ENDIAN__ */

